const mongodb = require('mongodb');
const { findByName, findById, updateObj, findByAttributes } = require('./common');

const { ObjectId } = mongodb;

function addDevice(deviceName, deviceId) {
  return {
    $addToSet: {
      devices: {
        deviceName,
        deviceId: ObjectId(deviceId),
      },
    },
  };
}

function findDevice(deviceId, state) {
  return {
    ...findById(deviceId),
    state,
  };
}

function findDevices(deviceIds, state) {
  const query = { state };
  if (Array.isArray(deviceIds)) {
    const deviceIdsObj = deviceIds.map(item => ObjectId(item));
    Object.assign(query, { _id: { $in: deviceIdsObj } });
  }
  return query;
}

function addExternalRef(objectData) {
  return { $push: { externalCoding: { ...objectData } } };
}

function addExternalRefAttribute(deviceId, externalSource, externalDetail) {
  const setObject = {};
  // eslint-disable-next-line no-restricted-syntax
  for (const [key, value] of Object.entries(externalDetail)) {
    const setKey = `externalCoding.$.externalDetail.${key}`;
    setObject[setKey] = value;
  }
  return [{
    _id: ObjectId(deviceId),
    externalCoding: { $elemMatch: { externalSource } },
  },
  { $set: setObject },
  ];
}

function deleteExternalRefAttribute(deviceId, externalSource, externalDetail) {
  return [{
    _id: ObjectId(deviceId),
    externalCoding: { $elemMatch: { externalSource } },
  },
  { $unset: { [`externalCoding.$.externalDetail.${externalDetail}`]: '' } },
  ];
}

function removeExternalRef(externalSource) {
  return { $pull: { externalCoding: { externalSource } } };
}

module.exports = {
  ObjectId,
  findDevice,
  findDevices,
  findByName,
  findById,
  updateObj,
  findByAttributes,
  addDevice,
  addExternalRef,
  addExternalRefAttribute,
  deleteExternalRefAttribute,
  removeExternalRef,
};
