const mongodb = require('mongodb');
const { findByName, findById, updateObj, findByAttributes } = require('./common');

const PROCESSOR_COLLECTION = 'productProccessors';

const { ObjectId } = mongodb;

function findProductByFamily(productFamilyId, name = null) {
  if (name) { return { productFamilyId, name }; }
  return { productFamilyId };
}

function findSubProdByProd(productId, name = null) {
  if (name) { return { productId, name }; }
  return { productId };
}

function addCategories(categories) {
  return {
    $addToSet: { categories: { $each: categories } },
  };
}

function addCaptureFields(captureFields) {
  return {
    $addToSet: { captureFields: { $each: captureFields } },
  };
}

function removeCaptureFields(CaptureFieldCode) {
  return (Array.isArray(CaptureFieldCode))
    ? { $pull: { captureFields: { code: { $in: CaptureFieldCode } } } }
    : { $pull: { captureFields: { code: CaptureFieldCode } } };
}

function addCaptureFieldOption(id, code, option) {
  return [{
    _id: ObjectId(id), captureFields: { $elemMatch: { code } },
  },
  {
    $addToSet: { 'captureFields.$.options': option },
  }];
}

function removeCaptureFieldOption(id, code, text) {
  return [{
    _id: ObjectId(id), captureFields: { $elemMatch: { code } },
  },
  {
    $pull: { 'captureFields.$.options': { text } },
  }];
}

function addAttributes(attributes) {
  const attributeObj = Object.entries(attributes).reduce(
    (acc, [key, value]) => Object.assign(acc, { [`attributes.${key}`]: value }), {},
  );
  return {
    $set: attributeObj,
  };
}

function removeAttributes(attributeName) {
  return {
    $unset: { [`attributes.${attributeName}`]: '' },
  };
}

function findEnableProd(productId, subproductId, groupId, thirdPartyId = null) {
  const query = {};
  if (productId) { query.productId = productId; }
  if (subproductId) { query.subproductId = subproductId; }
  if (groupId) {
    query.groupId = groupId;
  } else if (thirdPartyId) {
    query.thirdPartyId = thirdPartyId;
  }
  return query;
}

function findProduct(productId) {
  const query = [{
    $match: { _id: ObjectId(productId) },
  }, {
    $lookup: {
      from: PROCESSOR_COLLECTION,
      localField: 'proccessorId',
      foreignField: '_id',
      as: 'processor',
    },
  }, {
    $addFields: {
      processor: { $arrayElemAt: ['$processor', 0] },
    },
  }];
  return query;
}

module.exports = {
  findByName,
  findById,
  updateObj,
  findByAttributes,
  findProductByFamily,
  findSubProdByProd,
  addCategories,
  addCaptureFields,
  removeCaptureFields,
  addCaptureFieldOption,
  removeCaptureFieldOption,
  addAttributes,
  removeAttributes,
  findEnableProd,
  findProduct,
};
