const mongodb = require('mongodb');
const commonQueries = require('./common');

const { ObjectId } = mongodb;

function findStores(thirdPartyId, storeIds) {
  const query = {};
  if (Array.isArray(storeIds)) {
    const thirdIds = storeIds.map(item => ObjectId(item));
    Object.assign(query, { _id: { $in: thirdIds } });
  } else {
    Object.assign(query, { thirdPartyId: ObjectId(thirdPartyId) });
  }
  return query;
}

function addStoreToThird(storeName, storeId) {
  return {
    $addToSet: {
      stores: {
        storeName,
        storeId: ObjectId(storeId),
      },
    },
  };
}

function findStoreGroupMember(storeId, groupId) {
  return { storeId, groupId };
}

function findByUser(userId) {
  return { users: { userId: ObjectId(userId) } };
}

module.exports = {
  ...commonQueries,
  // Stores
  findStores,
  addStoreToThird,
  findStoreGroupMember,
  findByUser,
};
