const mongodb = require('mongodb');
const { findByName, findById, updateObj, findByAttributes } = require('./common');

const { ObjectId } = mongodb;

function findThirdParty(idDocType, idDocNumber) {
  return { idDocType, idDocNumber };
}

function findThirdparties(thirdPartyIds) {
  const query = {};
  if (Array.isArray(thirdPartyIds)) {
    const thirdIds = thirdPartyIds.map(item => ObjectId(item));
    Object.assign(query, { _id: { $in: thirdIds } });
  }
  return query;
}

function findRoleThirdParties(roleId) {
  return { roles: { $elemMatch: { roleId } } };
}

function addExternalRef(objectData) {
  return { $push: { externalCoding: { ...objectData } } };
}

function removeExternalRef(externalSource) {
  return { $pull: { externalCoding: { externalSource } } };
}

function findThirdRole(thirdPartyId, roleId) {
  return {
    _id: ObjectId(thirdPartyId),
    roles: { $elemMatch: { roleId: ObjectId(roleId) } },
  };
}

function addRole(roleData) {
  return { $push: { roles: { ...roleData } } };
}

function removeRole(roleId) {
  return { $pull: { roles: { roleId } } };
}

function findGroup(owner, groupName) {
  const ownerId = ObjectId(owner);
  return { ownerId, groupName };
}

function findOwnerGroups(ownerId) {
  return { ownerId };
}

const GROUP_COLLECTION = 'ThirdPartyGroups';

function findThirdPartyGroups(thirdParty) {
  const thirdPartyId = ObjectId(thirdParty);
  return [{
    $match: { thirdPartyId },
  },
  {
    $lookup: {
      from: GROUP_COLLECTION,
      localField: 'groupId',
      foreignField: '_id',
      as: 'groups',
    },
  },
  {
    $project: {
      groups: 1,
      _id: 0,
    },
  },
  {
    $unwind: { path: '$groups' },
  },
  {
    $project: {
      _id: '$groups._id',
      ownerId: '$groups.ownerId',
      groupName: '$groups.groupName',
      roleRelationId: '$groups.roleRelationId',
      exclusive: '$groups.exclusive',
      enabled: '$groups.enabled',
      creation: '$groups.creation',
    },
  },
  ];
}

function findThirdGroup(thirdPartyId, groupId) {
  return { thirdPartyId, groupId };
}

function findMembers(groupId) {
  return { groupId };
}

function findThirdRelation(primaryThirdParty, secondaryThirdParty, roleRelationId) {
  return { primaryThirdParty, secondaryThirdParty, roleRelationId };
}

function addUserReference(userName, device) {
  return { $addToSet: { users: { userName, device } } };
}

function findByUser(userName, device) {
  return { users: { userName, device } };
}

module.exports = {
  findByName,
  findById,
  updateObj,
  findByAttributes,
  findThirdParty,
  findThirdparties,
  findRoleThirdParties,
  addExternalRef,
  removeExternalRef,
  findThirdRole,
  addRole,
  removeRole,
  findGroup,
  findOwnerGroups,
  findThirdPartyGroups,

  findThirdGroup,
  findMembers,
  findThirdRelation,
  addUserReference,
  findByUser,
};
