const EnvironmentManager = require('./lib/ecosystem/environment');
const PriceManager = require('./lib/ecosystem/prices');
const ProductManager = require('./lib/ecosystem/products');
const ThirdPartyManager = require('./lib/ecosystem/third-parties');
const StoreManager = require('./lib/ecosystem/stores');
const SetupManager = require('./lib/ecosystem/setups');
const DeviceManager = require('./lib/ecosystem/devices');
const UserManager = require('./lib/ecosystem/users');
const { openPoolSet, getDb } = require('./lib/helpers/db-pool');
const {
  appDbPoolSet, dbQueries: {
    environment: envQueries,
    prices: priceQueries,
    products: productQueries,
    thirdParties: thirdpartyQueries,
    devices: deviceQueries,
    users: userQueries,
    stores: storeQueries,
    setups: setupQueries,
  },
} = require('./config');

const DEFAULT_DATABASE = 'ecosystem';

class EcosystemManager {
  constructor(databaseParams, logger, errMgr, privateKeyFile, databasePool = null) {
    this.logger = logger;
    this.errMgr = errMgr;
    this.initObjects(databaseParams, logger, errMgr, privateKeyFile, databasePool);
  }

  async initObjects(databaseParams, logger, errMgr, privateKeyFile, databasePool = null) {
    if (!databasePool) {
      await openPoolSet(databaseParams, logger, errMgr, privateKeyFile);
    } else {
      appDbPoolSet[DEFAULT_DATABASE] = databasePool;
    }
    this.environmentManager = new EnvironmentManager(getDb, logger, errMgr, envQueries);
    this.priceManager = new PriceManager(getDb, logger, errMgr, priceQueries);
    this.setupManager = new SetupManager(getDb, logger, errMgr, setupQueries);
    this.productManager = new ProductManager(getDb, logger, errMgr, productQueries);
    this.thirdPartyManager = new ThirdPartyManager(getDb, logger, errMgr, thirdpartyQueries);
    this.storeManager = new StoreManager(getDb, logger, errMgr, storeQueries);
    this.deviceManager = new DeviceManager(getDb, logger, errMgr, deviceQueries);
    this.userManager = new UserManager(getDb, logger, errMgr, userQueries);
  }
}
module.exports = EcosystemManager;
